use axum::{
    extract::{Path, State},
    http::HeaderMap,
    routing::post,
    Json, Router,
};
use bitflags::Flags;
use serde::Deserialize;

use crate::{
    auth::get_user_from_header,
    database::model::Message as DbMessage,
    model::{
        member::{Groups, Roles},
        message::{Channel, MessageCreate, MessageStatus},
        Message,
    },
    AppState,
};

pub fn routes() -> Router<AppState> {
    Router::new()
        .route("/messages", post(message_create))
        .route("/messages/{message_id}/send", post(message_send))
}

#[derive(Debug, Deserialize)]
pub struct MessageCreateRequest {
    title: String,
    content: String,
    channel: String,
    member_groups: String,
    member_roles: String,
}

pub async fn message_create(
    State(state): State<AppState>,
    headers: HeaderMap,
    Json(request): Json<MessageCreateRequest>,
) -> Result<String, crate::Error> {
    let user = get_user_from_header(&state.pool, &headers).await?;

    user.authorize(&state.pool, Some(Roles::ADMIN | Roles::MESSAGES), None)
        .await?;

    let db_message: DbMessage = Message::new(MessageCreate {
        title: request.title,
        content: request.content,
        channel: request.channel,
        member_groups: request.member_groups,
        member_roles: request.member_roles,
        scheduled_at: None,
        thumbnail_url: None,
    })?
    .into();

    let mut transaction = state.pool.begin().await?;

    db_message.insert(&mut transaction).await?;

    transaction.commit().await?;

    Ok(db_message.message_id.to_string())
}

pub async fn message_send(
    State(state): State<AppState>,
    headers: HeaderMap,
    Path(message_id): Path<String>,
) -> Result<(), crate::Error> {
    let user = get_user_from_header(&state.pool, &headers).await?;

    user.authorize(&state.pool, Some(Roles::ADMIN | Roles::MESSAGES), None)
        .await?;

    let message_id: uuid::Uuid =
        uuid::Uuid::parse_str(&message_id).map_err(|_| crate::Error::BadRequest {
            expected: "Could not convert message id to uuid".to_string(),
        })?;

    let mut transaction = state.pool.begin().await?;

    DbMessage::update_status(&mut transaction, &message_id, MessageStatus::Sent).await?;

    transaction.commit().await?;

    Ok(())
}
